/*      
   Tsunami for KallistiOS 1.1.9
        
   font.cpp

   (c)2002 Dan Potter
*/

#include <tsu/font.h>

CVSID("$Id: font.cpp,v 1.2 2002/09/28 08:00:34 bardtx Exp $");

Font::Font(const char *fn, int list) {
	m_font = plx_font_load(fn);
	assert( m_font != NULL );
	m_cxt = plx_fcxt_create(m_font, list);
	assert( m_cxt != NULL );
	m_a = m_r = m_g = m_b = 1.0f;
	m_ps = 24.0f;
}

Font::~Font() {
	plx_fcxt_destroy(m_cxt);
	plx_font_destroy(m_font);
}

void Font::setColor(float r, float g, float b) {
	m_r = r;
	m_g = g;
	m_b = b;
	plx_fcxt_setcolor4f(m_cxt, m_a, m_r, m_g, m_b);
}

void Font::setAlpha(float a) {
	m_a = a;
	plx_fcxt_setcolor4f(m_cxt, m_a, m_r, m_g, m_b);
}

void Font::setSize(float size) {
	m_ps = size;
	plx_fcxt_setsize(m_cxt, m_ps);
}

void Font::draw(float x, float y, float z, const char *text) {
	plx_fcxt_setpos(m_cxt, x, y, z);
	plx_fcxt_begin(m_cxt);
	plx_fcxt_draw(m_cxt, text);
	plx_fcxt_end(m_cxt);
}

void Font::drawCharBegin(float x, float y, float z) {
	plx_fcxt_setpos(m_cxt, x, y, z);
	plx_fcxt_begin(m_cxt);
}

float Font::drawChar(int ch) {
	return plx_fcxt_draw_ch(m_cxt, ch);
}

void Font::drawCharEnd() {
	plx_fcxt_end(m_cxt);
}

void Font::drawCentered(float x, float y, float z, const char *text) {
	centerCoords(text, &x, &y);
	draw(x, y, z, text);
}

void Font::smearDraw(float x, float y, float z, const char *text) {
	plx_fcxt_begin(m_cxt);

	// Draw the text itself
	plx_fcxt_setpos(m_cxt, x, y, z);
	plx_fcxt_draw(m_cxt, text);

	// Draw a nifty subtitle border
	// Guess all that time I spent on BakaSub wasn't wasted after all ^_-
	plx_fcxt_setcolor4f(m_cxt, m_a, 0.0f, 0.0f, 0.0f);
	plx_fcxt_setpos(m_cxt, x + 1, y + 0, z - 1.0f);
	plx_fcxt_draw(m_cxt, text);
	plx_fcxt_setpos(m_cxt, x + 1, y + 1, z - 1.0f);
	plx_fcxt_draw(m_cxt, text);
	plx_fcxt_setpos(m_cxt, x + 0, y + 1, z - 1.0f);
	plx_fcxt_draw(m_cxt, text);
	plx_fcxt_setpos(m_cxt, x - 1, y + 1, z - 1.0f);
	plx_fcxt_draw(m_cxt, text);
	plx_fcxt_setpos(m_cxt, x - 1, y + 0, z - 1.0f);
	plx_fcxt_draw(m_cxt, text);
	plx_fcxt_setpos(m_cxt, x - 1, y - 1, z - 1.0f);
	plx_fcxt_draw(m_cxt, text);
	plx_fcxt_setpos(m_cxt, x + 0, y + 0, z - 1.0f);
	plx_fcxt_draw(m_cxt, text);
		
	plx_fcxt_end(m_cxt);
	plx_fcxt_setcolor4f(m_cxt, m_a, m_r, m_g, m_b);
}

void Font::smearDrawCentered(float x, float y, float z, const char *text) {
	centerCoords(text, &x, &y);
	smearDraw(x, y, z, text);
}

void Font::getCharExtents(int c, float * l, float * u, float * r, float * d) {
	plx_fcxt_setsize(m_cxt, m_ps);
	plx_fcxt_char_metrics(m_cxt, c, l, u, r, d);
}

void Font::getTextSize(const char *text, float * w, float * h) {
	float left, right, bot, top;

	plx_fcxt_setsize(m_cxt, m_ps);
	plx_fcxt_str_metrics(m_cxt, text, &left, &top, &right, &bot);

	*w = left+right;
	*h = top+bot;
}

void Font::upperleftCoords(const char * text, float *x, float *y) {
	float left, right, bot, top;

	plx_fcxt_setsize(m_cxt, m_ps);
	plx_fcxt_str_metrics(m_cxt, text, &left, &top, &right, &bot);

	*y = *y + top;
}

void Font::centerCoords(const char * text, float *x, float *y) {
	float left, right, bot, top;

	plx_fcxt_setsize(m_cxt, m_ps);
	plx_fcxt_str_metrics(m_cxt, text, &left, &top, &right, &bot);

	*x = *x - (left+right)/2;
	*y = *y + (bot+top)/2;
}
